from django.db import models
from gubbins.db.queryset import InheritanceQuerySet

                
class InheritanceManager(models.Manager):
    def get_query_set(self):
        return InheritanceQuerySet(model=self.model, using=self._db)
    


import urlparse
import urllib
import random 

def random_models(qs, count):
    objects = []
    total_count = qs.count()
    ids = set()
    if total_count > 0:
        while len(objects) < min(count, total_count):
            idx = random.randint(0, total_count - 1)
            obj = qs[idx]
            if obj.id in ids:
                continue
            objects.append(obj)
            ids.add(obj.id)

    return objects


def append_params(url, params):
    url_parts = urlparse.urlparse(url)
    query = urlparse.parse_qsl(url_parts.query)
    for key, value in params.iteritems():
        query.append( (key, value) )
    new_url_parts = list(url_parts)
    new_url_parts[4] = urllib.urlencode(query)
    return urlparse.urlunparse(new_url_parts)




from django.db.models.fields.related import SingleRelatedObjectDescriptor
from django.db.models.query import QuerySet



# see http://jeffelmore.org/tag/django-python-inheritance-downcasting-orm-polymorphism-queryset/
class InheritanceQuerySet(QuerySet):
    """
    """
    
    def select_subclasses(self, *subclasses):
        if not subclasses:
            subclasses = [o for o in dir(self.model)
                          if isinstance(getattr(self.model, o), SingleRelatedObjectDescriptor)\
                          and issubclass(getattr(self.model,o).related.model, self.model)]
        new_qs = self.select_related(*subclasses)
        new_qs.subclasses = subclasses
        return new_qs

    def filter(self, *args, **kwargs): #@ReservedAssignment inherited method, can't rename
        qs = QuerySet.filter(self, *args, **kwargs)
        return qs.select_subclasses()

    def _clone(self, klass=None, setup=False, **kwargs):
        try:
            kwargs.update({'subclasses': self.subclasses})
        except AttributeError:
            pass
        return super(InheritanceQuerySet, self)._clone(klass, setup, **kwargs)
        
    def iterator(self):
        iterat = super(InheritanceQuerySet, self).iterator()
        if getattr(self, 'subclasses', False):
            for obj in iterat:
                obj = [getattr(obj, s) for s in self.subclasses if getattr(obj, s)] or [obj]
                yield obj[0]
        else:
            for obj in iterat:
                yield obj
                
                


from django.conf.urls import include


class ReusableAppURLs(object):
    
    def __init__(self, app_namespace, urlpatterns):
        self.app_namespace = app_namespace
        self.urlpatterns = urlpatterns

    def __call__(self, instance_namespace=None):
        instance_namespace = instance_namespace or self.app_namespace
        return include(self.urlpatterns, self.app_namespace, instance_namespace)
  
    


from django.core.exceptions import ImproperlyConfigured
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
import json
import re


meta_super_class = type
try:
    # Django version 1.3 used this class for metaclasses of fields,
    # which results in an error when trying to use the EnumMeta
    # metaclass instead:
    #
    #       metaclass conflict: the metaclass of a derived class must be 
    #       a (non-strict) subclass of the metaclasses of all its bases
    #
    # This is because using EnumMeta causes fields to have two 'metaclass'
    # definitions when using Django 1.3. The solution is to have EnumMeta
    # be a subclass of LegacyConnection, but this is only possible in 1.3
    # as in 1.4, Legacy Connection was removed
    from django.db.models.fields.subclassing import LegacyConnection
    meta_super_class = LegacyConnection
except ImportError:
    pass

class EnumMeta(meta_super_class):
    
    def __new__(cls, name, bases, attrs):
        inst = meta_super_class.__new__(cls, name, bases, attrs)
        inst.options = EnumMeta._get_options(inst)
        inst.values = EnumMeta._get_values(inst)
        inst.choices = zip(inst.values, inst.options)
        return inst
    
    @staticmethod
    def _get_options(inst):
        """
        Gets the list of possible options for this object.
        """
        options = []
        
        for attr_name in dir(inst):
            if attr_name.startswith('_'):
                # ignore any 'private' attributes
                continue
            if not re.match('^[A-Z][A-Z0-9_]+$', attr_name):
                # ignore anything which is not all uppercase
                continue
            attr = getattr(inst, attr_name)
            if callable(attr):
                # ignore anything which is not a simple value
                continue
            options.append(attr_name)
        
        return options

    @staticmethod
    def _get_values(inst):
        return [ getattr(inst, attr_name) for attr_name in inst.options ]



class EnumField(models.CharField):
    """
    
    Example usage::

      class SourceType(EnumField):
          WIKIPEDIA = 'w'
          BBC_NEWS = 'b'
          TWITTER = 't'
    
    Note that field names must be uppercase to be recognised
    as possible field values
    
    """
    __metaclass__ = EnumMeta
    
    def __init__(self, *args, **kwargs):
        kwargs = self._get_field_kwargs(kwargs)
        super(EnumField, self).__init__(*args, **kwargs)
    
    def __getattr__(self, name):
        if self.options is not None and name in self.options:
            return name
        raise AttributeError('No such enum option: %s' % name)
    
    def _get_field_kwargs(self, kwargs):
        max_length = max( map(len, map(str, self.values) ) )
        if 'max_length' in kwargs:
            if kwargs['max_length'] < max_length:
                raise ImproperlyConfigured('Supplied max_length is too short, enum values will be truncated! It should be %s or unspecified to be set automatically' % max_length)
        else:
            kwargs['max_length'] = max_length
        
        if 'choices' in kwargs:
            raise ImproperlyConfigured('"choices" keyword argument is not used as it is automatically generated')
        kwargs['choices'] = self.choices
        
        return kwargs
    
    def get_prep_value(self, value):
        if value not in [None, ''] + self.values:
            raise ValueError('%s is not an acceptable value for this field' % value)
        return super(EnumField, self).get_prep_value(value)
    
    def south_field_triple(self):
        # see http://south.readthedocs.org/en/latest/customfields.html#south-field-triple
        module = self.__class__.__module__
        field_name = self.__class__.__name__
        return ('%s.%s' % (module, field_name), [], {})




class JSONField(models.TextField):
    """
    JSONField is a generic textfield that neatly serializes/unserializes
    JSON objects seamlessly.
    Django snippet #1478

    example:
        class Page(models.Model):
            data = JSONField(blank=True, null=True)


        page = Page.objects.get(pk=5)
        page.data = {'title': 'test', 'type': 3}
        page.save()
    """
    # origially cribbed from django-annoying, which seems to have lain dormant
    # for a while and no pull requests are getting merged in

    __metaclass__ = models.SubfieldBase

    class CannotStoreTypeException(Exception):
        def __init__(self, type_):
            self._type = type_
        def __repr__(self):
            return 'Cannot store type in a JSONField: %s' % self._type

    def to_python(self, value):
        if value == "":
            return None

        try:
            if isinstance(value, basestring):
                return json.loads(value)
        except ValueError:
            pass 
        # TODO: this is shitty behaviour, should be fixed, but what should the
        # field do if the value in the database is invalid?
        return value

    def get_db_prep_save(self, value, *args, **kwargs):
        if value == "":
            return None
        
        # see what dummy key we need to use
        if not isinstance(value, dict):
            raise JSONField.CannotStoreTypeException(type(value))

        value = json.dumps(value, cls=DjangoJSONEncoder)
        return super(JSONField, self).get_db_prep_save(value, *args, **kwargs)
    
    def south_field_triple(self):
        # see http://south.readthedocs.org/en/latest/customfields.html#south-field-triple
        module = self.__class__.__module__
        field_name = self.__class__.__name__
        return ('%s.%s' % (module, field_name), [], {})
    



import time
from django.core.files.uploadhandler import FileUploadHandler

class SlowFileUploadHandler(FileUploadHandler):
    """
    This is an implementation of the Django file upload handler which will
    sleep between processing chunks in order to simulate a slow upload. This
    is intended for development when creating features such as an AJAXy
    file upload progress bar, as uploading to a local process is often too
    quick.
    """
    def receive_data_chunk(self, raw_data, start):
        time.sleep(2);
        return raw_data
    
    def file_complete(self, file_size):
        return None



import random


def random_models(qs, count):
    objects = []
    total_count = qs.count()
    ids = set()
    if total_count > 0:
        while len(objects) < min(count, total_count):
            idx = random.randint(0, total_count - 1)
            obj = qs[idx]
            if obj.id in ids:
                continue
            objects.append(obj)
            ids.add(obj.id)

    return objects


# -*- coding: UTF-8 -*-
from distutils.core import setup
from setuptools import find_packages

_version = "1.0.2"
_packages = find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"])
    
# common dependencies
_install_requires = [
            'django',
       ]

_long_description = """
django-gubbins is a collection of useful snippets for enhancing
or replacing functionality within Django. For more documentation, see the github
page at https://github.com/carlio/django-gubbins

Automatic downcasting to model subclasses :
This is for the case when you have a model hierarchy, with several models inheriting from
a base class, and you want to query the base class but get instances of the subclasses
out of the query set.

`EnumField` : provides an easy way to have a model field which only accepts 
some values, and at the same time makes it easy to reference those values in your code.

`JSONField` : allows you to store JSON strings in a database. 

`ReusableAppURLs` : a simple way to create the correct URL configuration for a reusable
django app.

`SlowFileUploadHandler` : can be used to really slow down handling of file uploads. 
This is an implementation of the Django file upload handler which will sleep between 
processing chunks in order to simulate a slow upload. This is intended for development 
when creating features such as an AJAXy file upload progress bar, as uploading to a 
local process is often too quick.
"""


setup( name='django-gubbins',
       url='https://github.com/carlio/django-gubbins',
       author='Carl Crowder',
       author_email='django-gubbins@jqx.be',
       description='A collection of useful snippets for enhancing or replacing functionality within Django',
       long_description=_long_description,
       version=_version,
       packages=_packages,
       install_requires=_install_requires,
       license='BSD',
       keywords = "django",
)
